#
# spec file for package galera-4
#
# Copyright (c) 2024 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define copyright Copyright 2007-2015 Codership Oy. All rights reserved. Use is subject to license terms under GPLv2 license.
%define docs %{_docdir}/%{name}
%define homedir   %{_localstatedir}/lib/garb
%define galeradir %{_localstatedir}/lib/galera
%define mariadb_version 10.5.10
%if ! %{defined _fillupdir}
  %define _fillupdir %{_localstatedir}/adm/fillup-templates
%endif
Name:           galera-4
Version:        26.4.17
Release:        1.3
Summary:        Galera: a synchronous multi-master wsrep provider (replication engine)
License:        GPL-2.0-only
Group:          Productivity/Databases/Tools
URL:            https://galeracluster.com/
Source:         https://releases.galeracluster.com/galera-4/source/%{name}-%{version}.tar.gz
Source1:        https://releases.galeracluster.com/galera-4/source/%{name}-%{version}.tar.gz.asc
Source2:        garb-user.conf
Patch0:         galera-3-25.3.10_fix_startup_scripts.patch
Patch2:         fix-cmake-install.patch
BuildRequires:  boost-devel
BuildRequires:  check-devel
BuildRequires:  gcc-c++
BuildRequires:  glibc-devel
# for fileownership of galeradir
BuildRequires:  mariadb >= %{mariadb_version}
BuildRequires:  pkgconfig
BuildRequires:  sysuser-tools
%sysusers_requires
BuildRequires:  pkgconfig(systemd)
Requires:       %{name}-wsrep-provider
Conflicts:      galera-3
%if 0%{?suse_version} >= 1500
BuildRequires:  libboost_program_options-devel
BuildRequires:  libboost_system-devel
%endif
BuildRequires:  cmake
# for fileownership of galeradir
%if %{with split_package}
BuildRequires:  mysql-server
%endif
%if 0%{?suse_version} >= 1500
BuildRequires:  libopenssl-devel
%else
BuildRequires:  openssl-devel
%endif
%if %{defined fedora}
BuildRequires:  python
%endif
Requires(post): %fillup_prereq

%description
Galera is a fast synchronous multimaster wsrep provider (replication engine)
for transactional databases and similar applications. For more information
about wsrep API see http://launchpad.net/wsrep. For a description of Galera
replication engine see http://www.codership.com.

%{copyright}

This software comes with ABSOLUTELY NO WARRANTY. This is free software,
and you are welcome to modify and redistribute it under the GPLv2 license.

%package wsrep-provider
Summary:        Galera support library
Group:          Productivity/Databases/Tools
URL:            https://galeracluster.com/
Requires:       mariadb-galera >= 10.4
# already required in mariadb-galera
Recommends:     socat
# for old setups. newer setups will need mariadbbackup
Recommends:     xtrabackup
Conflicts:      galera-3-wsrep-provider
Conflicts:      mariadb < 10.4

%description wsrep-provider
Galera is a fast synchronous multimaster wsrep provider (replication engine)
for transactional databases and similar applications. For more information
about wsrep API see http://launchpad.net/wsrep. For a description of Galera
replication engine see http://www.codership.com.

This package provides the libgalera_smm library.

%prep
%autosetup -p1

%build
%sysusers_generate_pre %{SOURCE2} garb garb-user.conf
export CFLAGS="%{optflags}"
export CXXFLAGS="%{optflags}"
%if 0%{?suse_version} > 1320
export CFLAGS="$CFLAGS -Wno-implicit-fallthrough"
export CXXFLAGS="$CXXFLAGS -Wno-implicit-fallthrough"
%endif
%cmake -DGALERA_SYSTEMD_UNITDIR:PATH=%{_unitdir} \
       -DCMAKE_INSTALL_LIBEXECDIR:PATH=%{_libexecdir} \
       -DCMAKE_INSTALL_DOCDIR:PATH=%{_datarootdir}/doc/packages/%{name}
%cmake_build

%install
%cmake_install
mkdir -p %{buildroot}%{_sysusersdir}
install -D -m 0644 %{SOURCE2} %{buildroot}%{_sysusersdir}/

mkdir -p %{buildroot}%{_fillupdir}
mv %{buildroot}%{_prefix}/etc/garb.cnf %{buildroot}%{_fillupdir}/sysconfig.garb

install -D -d -m 0750 %{buildroot}%{homedir} %{buildroot}%{galeradir}
install -D -d -m 0755 %{buildroot}%{_sysconfdir}/my.cnf.d/
cat > %{buildroot}%{_sysconfdir}/my.cnf.d/51-%{name}-wsrep-provider.cnf <<EOF
# All changes to this file will be overwritten with the next package update.
# For configuring galera please use 50-galera.cnf or another file
# This file is only here to set the proper path to the wsrep_provider library
[mysqld]
wsrep_provider=%{_libdir}/libgalera_smm.so
EOF

sed -e 's;/usr/libexec;%{_libexecdir};' -i %{buildroot}%{_unitdir}/garb.service

%files
# common
%doc %{docs}
# /common
# garb
%{_unitdir}/garb.service
%{_libexecdir}/garb-systemd
%config(noreplace,missingok) %{_fillupdir}/sysconfig.garb
%{_bindir}/garbd
#
%{_mandir}/man8/garbd.8%{?ext_man}
#
%dir %attr(0750,garb,garb) %{homedir}
%{_sysusersdir}/garb-user.conf
# /garb
# plugin
%if %{with split_package}
%dir %attr(0750,mysql,mysql) %{galeradir}
%endif
# plugin

%files wsrep-provider
%{_libdir}/libgalera_smm.so
%config %{_sysconfdir}/my.cnf.d/51-%{name}-wsrep-provider.cnf

%pre -f garb.pre
%service_add_pre garb.service

%preun
%service_del_preun garb.service

%post
%service_add_post garb.service
%fillup_only -n garb

%postun
%service_del_postun garb.service

%changelog
* Mon Feb 19 2024 Danilo Spinella <danilo.spinella@suse.com>
- Update to 26.4.17:
  - "Zero-level" certification keys are implemented. Currently there are 3 levels
  of resource keys: schema, table, row, yet one common resource used by every
  writeset, a node. The key is added by default to every writeset with the default
  type WSREP_KEY_SHARED. Now, node-wide operations can be performed with the same
  semantics as regular transactions.
  - Non-SYNCED nodes are not counted when calculating commit cut - maximum seqno
  that is no longer needed in certification index; situations may arise when group
  commit cut is applied before all preceding events are committed on a non-SYNCED
  node, now drain all monitors to ensure that preceding events are committed.
  - Stability of node joining process for gmcast improved when new nodes are
  connecting concurrently as nodes previously may arrive at a different conclusion
  on which nodes will be blocked, however now, connections are explicitly allowed
  from all nodes which may be accepted as members in the next view.
  - Stability of node joining process for EVS improved by dropping EVS join
  messages without own identified in joining state, to avoid starting membership
  protocol before at least one of the existing nodes has seen at least one join
  message from a joiner.
  - Improve "Failed to report last committed" warning to make it more accurate and
  user-friendly.
- Update to 26.4.16:
  - Removed out-of-order certification for BF aborted transactions.
  - Better error handling when the socket is closed, also cleaned up excessive
  debug logging.
- Update to 26.4.15:
  - an assertion is relaxed when Galera is configured with commit order disabled
  - in certificate index preload, an assertion in handle_ist_trx_preload() fails
  if the first handled IST preload write set has failed certification, and
  incrementing the index position caused the certificate index to end up with the
  wrong position; now fixed by appending dummy writeset sequence numbers into
  Certification::trx_map_
  - ASAN reported memory leak if gcs is closed after becoming joined but before
  reaching synced
* Thu May  4 2023 Marcus Rueckert <mrueckert@suse.de>
- Update to 26.4.14:
  - memory leak in the gcs gcomm backend fixed (tp_ object deleted
    in GCommConn destructor)
  - protonet.backend option deprecated since it only supports one
    option, asio, and the option will be removed in future release.
    Related, protonet.version is also deprecated
  - deprecation message for socket.ssl_compression as SSL
    compression cannot be enabled and the option will be removed in
    future releases
  - library could parse incorrect parameters as long as it had a
    "good" prefix, i.e. evs.*, so fix prevents setting of invalid
    option values
  - parsing of ISO8601 durations previously accepted invalid
    values, now fixed
  - in addition to ISO8601 format, parsing durations now supports
    real number representation
* Wed Dec  7 2022 Marcus Rueckert <mrueckert@suse.de>
- Update to 26.4.13:
  - Complete IO for client handshake before starting an
    asynchronous read to fix an occasional connection failure when
    establishing new cluster connections
  - EOF-while-reading errors now suppressed when using OpenSSL 3.0
* Thu Jun  9 2022 Danilo Spinella <danilo.spinella@suse.com>
- Update to 26.4.12:
  * WORK_DIR parameter added for garb.cnf and launch scripts to ensure Galera
    Arbitrator handles recovery files better (this includes command line option to
    specify working directory), fixing codership/galera#313
  * Fixes to renaming fc_master_slave to fc_single_primary
  * Catch local_endpoint() and remote_endpoint() exceptions, so they aren't
    propagated to the stack, fixing a cluster crash and MDEV-25068
  * SSL options are explicitly initialised in garbd, fixing MDEV-28158
* Thu Apr 28 2022 Danilo Spinella <danilo.spinella@suse.com>
- Fill sysconfig configuration file
* Thu Apr 28 2022 Thorsten Kukuk <kukuk@suse.com>
- Fix sysusers generation
* Wed Apr 27 2022 Danilo Spinella <danilo.spinella@suse.com>
- Commit sed changes to file
- Use default shell for garb user ("-" which is replaced by /usr/sbin/nologin)
  * garb-user.conf
- Add sysusers macro
* Fri Feb 18 2022 Danilo Spinella <danilo.spinella@suse.com>
- Update to 26.4.11
- Remove upstreamed fix-cmake-build.patch
- Refresh fix-cmake-install.patch
* Thu Feb 17 2022 Danilo Spinella <danilo.spinella@suse.com>
- Fix build and systemd service for SLE-15-SP3
* Wed Jan 26 2022 Danilo Spinella <danilo.spinella@suse.com>
- Use cmake by default to fix build with GCC 12, fixes bsc#1194790
  * Add fix-cmake-install.patch
- Add fix-cmake-build.patch to fix cmake build, fixes bsc#1194827
* Thu Dec 30 2021 Danilo Spinella <danilo.spinella@suse.com>
- Migrate to sysusers for creating garb user
* Thu Nov 18 2021 Danilo Spinella <danilo.spinella@suse.com>
- Update to 26.4.10:
  http://releases.galeracluster.com/galera-4/release-notes-galera-26.4.10.txt
- Clean spec file by only allowing building for SLE >= 12 with systemd
- Run spec-cleaner
* Mon Sep 27 2021 Marcus Rueckert <mrueckert@suse.de>
- pass the version to scons to make the testsuite pass
- use system version of libraries if possible and enable ssl
  support explicitely
- prepare building with cmake. but it is not usable yet
* Mon Sep 27 2021 Marcus Rueckert <mrueckert@suse.de>
- update to 26.4.9
  http://releases.galeracluster.com/galera-4/release-notes-galera-26.4.9.txt
* Wed Dec  2 2020 Marcus Rueckert <mrueckert@suse.de>
- refreshed galera-3-25.3.10_fix_startup_scripts.patch
* Wed Dec  2 2020 Marcus Rueckert <mrueckert@suse.de>
- update to 26.4.6
  http://releases.galeracluster.com/galera-4/release-notes-galera-26.4.6.txt
* Thu Jul  2 2020 Marcus Rueckert <mrueckert@suse.de>
- allow also newer versions of mariadb
* Mon Apr  6 2020 Marcus Rueckert <mrueckert@suse.de>
- also conflict with the galera 3 packages
* Thu Feb 13 2020 Marcus Rueckert <mrueckert@suse.de>
- do not lock openssl to 1.0 anymore
* Thu Feb 13 2020 Marcus Rueckert <mrueckert@suse.de>
- initial package
